(*************************************************************
 **  MySQL Interface (libmysql.dll)
 **
 **  Converted by:
 **        Rangel Reale       hitnrun@geocities.com
 **
 **  Based on work from:
 **        Bob Silva          bsilva@umesd.k12.or.us
 **************************************************************)
unit libmysql;

interface

uses
  Windows;
  
// General declarations
const
  NAME_LEN                           =      64;
  LOCAL_HOST                         =      'localhost';
  LOCAL_HOST_NAMED_PIPE              =      '.';
  MYSQL_PORT                         =      3306;
  MYSQL_UNIX_ADDR                    =      '/tmp/mysql.sock';
  MYSQL_NAMEDPIPE                    =      'MySQL';
  PROTOCOL_VERSION                   =      10;
  MYSQL_SERVER_VERSION               =      '3.21.18-beta';
  FRM_VER                            =      6;
  MYSQL_VERSION_ID                   =      32115;

// MySQL types
type
  my_bool = Byte;
  gptr    = PChar;
  socket  = Word;

// ENUM - enum_server_command
type
  enum_server_command = Byte;
const
  COM_SLEEP                  = 0;
  COM_QUIT                   = 1;
  COM_INIT_DB                = 2;
  COM_QUERY                  = 3;
  COM_FIELD_LIST             = 4;
  COM_CREATE_DB              = 5;
  COM_DROP_DB                = 6;
  COM_REFRESH                = 7;
  COM_SHUTDOWN               = 8;
  COM_STATISTICS             = 9;
  COM_PROCESS_INFO           = 10;
  COM_CONNECT                = 11;
  COM_PROCESS_KILL           = 12;
  COM_DEBUG                  = 13;
  COM_PING                   = 14;
  COM_TIME                   = 15;

// Flags
const
   NOT_NULL_FLAG	     = 1;		(* Field can't be NULL *)
   PRI_KEY_FLAG	             = 2;		(* Field is part of a primary key *)
   UNIQUE_KEY_FLAG           = 4;		(* Field is part of a unique key *)
   MULTIPLE_KEY_FLAG         = 8;		(* Field is part of a key *)
   BLOB_FLAG	             = 16;		(* Field is a blob *)
   UNSIGNED_FLAG	     = 32;		(* Field is unsigned *)
   ZEROFILL_FLAG	     = 64;		(* Field is zerofill *)
   BINARY_FLAG	             = 128;
   (* The following are only sent to new clients *)
   ENUM_FLAG	             = 256;		(* field is an enum *)
   AUTO_INCREMENT_FLAG       = 512;		(* field is a autoincrement field *)
   TIMESTAMP_FLAG	     = 1024;		(* Field is a timestamp *)
   PART_KEY_FLAG	     = 16384;		(* Intern; Part of some key *)
   GROUP_FLAG	             = 32768;		(* Intern group field *)

// Refresh options
const
   REFRESH_GRANT		= 1;	        (* Refresh grant tables *)
   REFRESH_LOG		        = 2;	        (* Start on new log file *)
   REFRESH_TABLES		= 4;	        (* close all tables *)
   REFRESH_HOSTS		= 8;	        (* Flush host cache *)
   REFRESH_FAST		        = 32768;        (* Intern flag *)

// Client options
const
   CLIENT_LONG_PASSWORD	        = 1;	(* new more secure passwords *)
   CLIENT_FOUND_ROWS	        = 2;	(* Found instead of affected rows *)
   CLIENT_LONG_FLAG	        = 4;	(* Get all column flags *)
   CLIENT_CONNECT_WITH_DB	= 8;	(* One can specify db on connect *)
   CLIENT_NO_SCHEMA	        = 16;	(* Don't allow database.table.column *)
   CLIENT_COMPRESS		= 32;	(* Can use compression protcol *)
   CLIENT_ODBC		        = 64;	(* Odbc client *)
   CLIENT_LOCAL_FILES	        = 128;	(* Can use LOAD DATA LOCAL *)
   CLIENT_IGNORE_SPACE	        = 256;	(* Ignore spaces before '(' *)
   
const
   MYSQL_ERRMSG_SIZE            = 200;
   NET_READ_TIMEOUT             = 30;			(* Timeout on read *)
   NET_WRITE_TIMEOUT            = 60;			(* Timeout on write *)
   NET_WAIT_TIMEOUT             = 8*60*60;		(* Wait for new query *)
   
// ENUM - enum_net_type
type
  enum_net_type = Integer;
const
  NET_TYPE_TCPIP                = 0;
  NET_TYPE_SOCKET               = 1;
  NET_TYPE_NAMEDPIPE            = 2;

// STRUCT - st_net
type
  NET = record
          nettype: enum_net_type;
          hPipe: Integer;
          fd: Socket;     // Socket
          fcntl: Integer;
          buff, buff_end, write_pos, read_pos: PChar;
          last_error: array[0..MYSQL_ERRMSG_SIZE-1] of Char;
          last_errno, max_packet, timeout, pkt_nr: DWORD;
          error, return_errno, compress: my_bool;
          remain_in_buf, length, buf_length, where_b: DWORD;
          more: my_bool;
          save_char: Char;
        end;

const
  packet_error            = -1;
  
// ENUM - enum_field_types
type
  enum_field_types = Byte;
const
  FIELD_TYPE_DECIMAL             = 0;
  FIELD_TYPE_TINY                = 1;
  FIELD_TYPE_SHORT               = 2;
  FIELD_TYPE_LONG                = 3;
  FIELD_TYPE_FLOAT               = 4;
  FIELD_TYPE_DOUBLE              = 5;
  FIELD_TYPE_NULL                = 6;
  FIELD_TYPE_TIMESTAMP           = 7;
  FIELD_TYPE_LONGLONG            = 8;
  FIELD_TYPE_INT24               = 9;
  FIELD_TYPE_DATE                = 10;
  FIELD_TYPE_TIME                = 11;
  FIELD_TYPE_DATETIME            = 12;
  FIELD_TYPE_YEAR                = 13;
  FIELD_TYPE_NEWDATE             = 14;
  FIELD_TYPE_ENUM                = 247;
  FIELD_TYPE_SET                 = 248;
  FIELD_TYPE_TINY_BLOB           = 249;
  FIELD_TYPE_MEDIUM_BLOB         = 250;
  FIELD_TYPE_LONG_BLOB           = 251;
  FIELD_TYPE_BLOB                = 252;
  FIELD_TYPE_VAR_STRING          = 253;
  FIELD_TYPE_STRING              = 254;
  FIELD_TYPE_CHAR                = FIELD_TYPE_TINY;		(* For compability *)
  FIELD_TYPE_INTERVAL            = FIELD_TYPE_ENUM;	        (* For compability *)
  
// Defines
function IS_PRI_KEY(n: Integer): Boolean;
function IS_NOT_NULL(n: Integer): Boolean;
function IS_BLOB(n: Integer): Boolean;
function IS_NUM(t: Integer): Boolean;

// STRUCT - MYSQL_FIELD
type
  PMYSQL_FIELD = ^MYSQL_FIELD;
  MYSQL_FIELD = record
                  name: PChar;                        (* Name of column *)
                  table: PChar;                       (* Table of column if column was a field *)
                  def: PChar;                         (* Default value (set by mysql_list_fields) *)
                  fieldtype: enum_field_types;        (* Type of field. Se mysql_com.h for types *)
                  length: DWORD;                      (* Width of column *)
                  max_length: DWORD;                  (* Max width of selected set *)
                  flags: DWORD;                       (* Div flags *)
                  decimals: DWORD;                    (* Number of decimals in field *)
                end;

// STRUCT - USED_MEM
type
  PUSED_MEM = ^USED_MEM;
  USED_MEM = record                            (* struct for once_alloc *)
               next: PUSED_MEM;                       (* Next block in use *)
               left: DWORD;                           (* memory left in block  *)
               size: DWORD;                           (* size of block *)
             end;

// STRUCT - MEM_ROOT
type
  PMEM_ROOT = ^MEM_ROOT;
  MEM_ROOT = record
               free: PUSED_MEM;
               used: PUSED_MEM;
               min_malloc: DWORD;
               block_size: DWORD;
               error_handler: Pointer;      // procedure()
             end;

// MYSQL_ROW
type
  CAST_MYSQL_ROW = array[0..$FFFF] of PChar;
  MYSQL_ROW = ^PChar;

type
  MYSQL_FIELD_OFFSET = DWORD;
  my_ulonglong = Int64;

// STRUCT - MYSQL_ROWS
type
  PMYSQL_ROWS = ^MYSQL_ROWS;
  MYSQL_ROWS = record
                 next: PMYSQL_ROWS;
                 data: MYSQL_ROW;
               end;
  MYSQL_ROW_OFFSET = MYSQL_ROWS;
  
// STRUCT - MYSQL_DATA
type
  PMYSQL_DATA = ^MYSQL_DATA;
  MYSQL_DATA = record
                 rows: my_ulonglong;
                 fields: DWORD;
                 data: PMYSQL_ROWS;
                 alloc: MEM_ROOT;
               end;
               
// STRUCT - st_mysql_options
type
  st_mysql_options = record
                       connect_timeout: DWORD;
                       compress, named_pipe: my_bool;
                     end;
                     
// ENUM - mysql_option
type
  mysql_option = Byte;
const
  MYSQL_OPT_CONNECT_TIMEOUT                  = 0;
  MYSQL_OPT_COMPRESS                         = 1;
  MYSQL_OPT_NAMED_PIPE                       = 2;

// ENUM - mysql_status
type
  mysql_status = Byte;
const
  MYSQL_STATUS_READY                         = 0;
  MYSQL_STATUS_GET_RESULT                    = 1;
  MYSQL_STATUS_USE_RESULT                    = 2;
  
// STRUCT - MYSQL
type
  PMYSQL = ^MYSQL;
  MYSQL = record
            net: NET;
            host, user, passwd, unix_socket, server_version, host_info, info, db: PChar;
            port,client_flag,server_capabilities: DWORD;
            protocol_version: DWORD;
            field_count: DWORD;
            thread_id: DWORD;		        (* Id for connection in server *)
            affected_rows: my_ulonglong;
            insert_id: my_ulonglong;		(* id if insert on table with NEXTNR *)
            extra_info: my_ulonglong;		(* Used by mysqlshow *)
            packet_length: DWORD;
            status: mysql_status;
            fields: PMYSQL_FIELD;
            field_alloc: MEM_ROOT;
            free_me: my_bool;		        (* If free in mysql_close *)
            reconnect: my_bool;		        (* set to 1 if automatic reconnect *)
            options: st_mysql_options;
          end;

// STRUCT - MYSQL_RES
type
  PMYSQL_RES = ^MYSQL_RES;
  MYSQL_RES = record
                row_count: my_ulonglong;
                field_count, current_field: DWORD;
                fields: PMYSQL_FIELD;
                data: PMYSQL_DATA;
                data_cursor: PMYSQL_ROWS;
                field_alloc: MEM_ROOT;
                row: MYSQL_ROW;			(* If unbuffered read *)
                current_row: MYSQL_ROW;		(* buffer to current row *)
                lengths: PDWORD;		(* column lengths of current row *)
                handle: PMYSQL;		        (* for unbuffered reads *)
                eof: my_bool;			(* Used my mysql_fetch_row *)
              end;
              
// DLL functions
function mysql_init(mysql: PMYSQL): PMYSQL; stdcall;
function mysql_connect(mysql: PMYSQL; host, user, passwd: PChar): PMYSQL; stdcall;
{function mysql_real_connect(mysql: PMYSQL; host, user, passwd, db: PChar; port: DWORD,
   unix_socket: PChar; clientflag: DWORD): PMYSQL; stdcall;}
function mysql_real_connect(mysql: PMYSQL; host, user, passwd: PChar; port: DWORD;
   unix_socket: PChar; clientflag: DWORD): PMYSQL; stdcall;
procedure mysql_close(sock: PMYSQL); stdcall;
function mysql_select_db(mysql: PMYSQL; db: PChar): Integer; stdcall;
function mysql_query(mysql: PMYSQL; q: PChar): Integer; stdcall;
function mysql_real_query(mysql: PMYSQL; q: PChar; length: DWORD): Integer; stdcall;
function mysql_create_db(mysql: PMYSQL; DB: PChar): Integer; stdcall;
function mysql_drop_db(mysql: PMYSQL; DB: PChar): Integer; stdcall;
function mysql_shutdown(mysql: PMYSQL): Integer; stdcall;
function mysql_dump_debug_info(mysql: PMYSQL): Integer; stdcall;
function mysql_refresh(mysql: PMYSQL; refresh_options: DWORD): Integer; stdcall;
function mysql_kill(mysql: PMYSQL; pid: DWORD): Integer; stdcall;
function mysql_ping(mysql: PMYSQL): Integer; stdcall;
function mysql_stat(mysql: PMYSQL): PChar; stdcall;
function mysql_get_server_info(mysql: PMYSQL): PChar; stdcall;
function mysql_get_client_info: PChar; stdcall;
function mysql_get_host_info(mysql: PMYSQL): PChar; stdcall;
function mysql_get_proto_info(mysql: PMYSQL): DWORD; stdcall;
function mysql_list_dbs(mysql: PMYSQL; wild: PChar): PMYSQL_RES; stdcall;
function mysql_list_tables(mysql: PMYSQL; wild: PChar): PMYSQL_RES; stdcall;
function mysql_list_fields(mysql: PMYSQL; table: PChar; wild: PChar): PMYSQL_RES; stdcall;
function mysql_list_processes(mysql: PMYSQL): PMYSQL_RES; stdcall;
function mysql_store_result(mysql: PMYSQL): PMYSQL_RES; stdcall;
function mysql_use_result(mysql: PMYSQL): PMYSQL_RES; stdcall;
function mysql_options(mysql: PMYSQL; option: mysql_option; arg: PChar): Integer; stdcall;
procedure mysql_free_result(result: PMYSQL_RES); stdcall;
procedure mysql_data_seek(result: PMYSQL_RES; offset: DWORD); stdcall;
function mysql_row_seek(result: PMYSQL_RES; offset: MYSQL_ROW_OFFSET): MYSQL_ROW_OFFSET; stdcall;
function mysql_field_seek(result: PMYSQL_RES; offset: MYSQL_FIELD_OFFSET): MYSQL_FIELD_OFFSET; stdcall;
function mysql_fetch_row(result: PMYSQL_RES): MYSQL_ROW; stdcall;
function mysql_fetch_lengths(result: PMYSQL_RES): PDWORD; stdcall;
function mysql_fetch_field(result: PMYSQL_RES): PMYSQL_FIELD; stdcall;
function mysql_escape_string(_to: PChar; from: PChar; from_length: DWORD): DWORD; stdcall;
procedure mysql_debug(debug: PChar); stdcall;

// #defined functions
function mysql_num_rows(res: PMYSQL_RES): Integer;
function mysql_num_fields(res: PMYSQL_RES): Integer;
function mysql_eof(res: PMYSQL_RES): my_bool;
function mysql_fetch_field_direct(res: PMYSQL_RES; fieldnr: DWORD): PMYSQL_FIELD;
function mysql_fetch_fields(res: PMYSQL_RES): PMYSQL_FIELD;
function mysql_row_tell(res: PMYSQL_RES): DWORD;
function mysql_field_tell(res: PMYSQL_RES): MYSQL_FIELD_OFFSET;
function mysql_affected_rows(mysql: PMYSQL): my_ulonglong;
function mysql_insert_id(mysql: PMYSQL): my_ulonglong;
function mysql_error(mysql: PMYSQL): PChar;
function mysql_errno(mysql: PMYSQL): DWORD;
function mysql_info(mysql: PMYSQL): PChar;
function mysql_reload(mysql: PMYSQL): Integer;
function mysql_thread_id(mysql: PMYSQL): DWORD;

implementation

const
  MYSQLDLL = 'libmysql.dll';
  
// Defines
function IS_PRI_KEY(n: Integer): Boolean;
begin
     Result := n and PRI_KEY_FLAG = PRI_KEY_FLAG;
end;

function IS_NOT_NULL(n: Integer): Boolean;
begin
     Result := n and NOT_NULL_FLAG = NOT_NULL_FLAG;
end;

function IS_BLOB(n: Integer): Boolean;
begin
     Result := n and BLOB_FLAG = BLOB_FLAG;
end;

function IS_NUM(t: Integer): Boolean;
begin
     Result := (t <= FIELD_TYPE_INT24) or (t = FIELD_TYPE_YEAR);
end;

// DLL functions
function mysql_init; external MYSQLDLL;
function mysql_connect; external MYSQLDLL;
function mysql_real_connect; external MYSQLDLL;
procedure mysql_close; external MYSQLDLL;
function mysql_select_db; external MYSQLDLL;
function mysql_query; external MYSQLDLL;
function mysql_real_query; external MYSQLDLL;
function mysql_create_db; external MYSQLDLL;
function mysql_drop_db; external MYSQLDLL;
function mysql_shutdown; external MYSQLDLL;
function mysql_dump_debug_info; external MYSQLDLL;
function mysql_refresh; external MYSQLDLL;
function mysql_kill; external MYSQLDLL;
function mysql_ping; external MYSQLDLL;
function mysql_stat; external MYSQLDLL;
function mysql_get_server_info; external MYSQLDLL;
function mysql_get_client_info; external MYSQLDLL;
function mysql_get_host_info; external MYSQLDLL;
function mysql_get_proto_info; external MYSQLDLL;
function mysql_list_dbs; external MYSQLDLL;
function mysql_list_tables; external MYSQLDLL;
function mysql_list_fields; external MYSQLDLL;
function mysql_list_processes; external MYSQLDLL;
function mysql_store_result; external MYSQLDLL;
function mysql_use_result; external MYSQLDLL;
function mysql_options; external MYSQLDLL;
procedure mysql_free_result; external MYSQLDLL;
procedure mysql_data_seek; external MYSQLDLL;
function mysql_row_seek; external MYSQLDLL;
function mysql_field_seek; external MYSQLDLL;
function mysql_fetch_row; external MYSQLDLL;
function mysql_fetch_lengths; external MYSQLDLL;
function mysql_fetch_field; external MYSQLDLL;
function mysql_escape_string; external MYSQLDLL;
procedure mysql_debug; external MYSQLDLL;

// #defined functions
function mysql_num_rows(res: PMYSQL_RES): Integer;
begin
     Result := res^.row_count
end;

function mysql_num_fields(res: PMYSQL_RES): Integer;
begin
     Result := res^.field_count;
end;

function mysql_eof(res: PMYSQL_RES): my_bool;
begin
     Result := res^.eof
end;

function mysql_fetch_field_direct(res: PMYSQL_RES; fieldnr: DWORD): PMYSQL_FIELD;
begin
     Result := res^.fields;
     Inc(Result, fieldnr);
end;

function mysql_fetch_fields(res: PMYSQL_RES): PMYSQL_FIELD;
begin
     Result := res^.fields;
end;

function mysql_row_tell(res: PMYSQL_RES): DWORD;
begin
     Result := DWORD(res^.data_cursor);
end;

function mysql_field_tell(res: PMYSQL_RES): MYSQL_FIELD_OFFSET;
begin
     Result := res^.current_field;
end;

function mysql_affected_rows(mysql: PMYSQL): my_ulonglong;
begin
     Result := mysql^.affected_rows;
end;

function mysql_insert_id(mysql: PMYSQL): my_ulonglong;
begin
     Result := mysql^.insert_id;
end;

function mysql_error(mysql: PMYSQL): PChar;
begin
     Result := mysql^.net.last_error;
end;

function mysql_errno(mysql: PMYSQL): DWORD;
begin
     Result := mysql^.net.last_errno;
end;

function mysql_info(mysql: PMYSQL): PChar;
begin
     Result := mysql^.info;
end;

function mysql_reload(mysql: PMYSQL): Integer;
begin
     Result := mysql_refresh(mysql, REFRESH_GRANT);
end;

function mysql_thread_id(mysql: PMYSQL): DWORD;
begin
     Result := mysql^.thread_id;
end;

end.
